@@ -18,14 +18,12 @@
 #include <time.h>
 
 #ifdef HAVE_SSL
-/* this entire file is rather useless when you don't have
- * crypto...
- */
 #include <openssl/ssl.h>
 #include <openssl/evp.h>
 #include <openssl/rand.h>
 #include <openssl/err.h>
 #include <openssl/md5.h>
+#endif
 
 ldns_rr *
 ldns_dnssec_get_rrsig_for_name_and_type(const ldns_rdf *name,
@@ -169,7 +167,7 @@ ldns_dnssec_nsec3_closest_encloser(ldns_rdf *qname,
 		} else if (exact_match_found && flag) {
 			result = ldns_rdf_clone(sname);
 		} else if (exact_match_found && !flag) {
-			// error!
+			/* error! */
 			ldns_rdf_deep_free(hashed_sname);
 			goto done;
 		} else {
@@ -321,6 +319,7 @@ uint16_t ldns_calc_keytag_raw(uint8_t* key, size_t keysize)
 	}
 }
 
+#ifdef HAVE_SSL
 DSA *
 ldns_key_buf2dsa(ldns_buffer *key)
 {
@@ -425,6 +424,7 @@ ldns_key_buf2rsa_raw(unsigned char* key, size_t len)
 
 	return rsa;
 }
+#endif /* HAVE_SSL */
 
 ldns_rr *
 ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
@@ -453,24 +453,19 @@ ldns_key_rr2ds(const ldns_rr *key, ldns_hash h)
 	switch(h) {
 	default:
 	case LDNS_SHA1:
-		digest = LDNS_XMALLOC(uint8_t, SHA_DIGEST_LENGTH);
+		digest = LDNS_XMALLOC(uint8_t, LDNS_SHA1_DIGEST_LENGTH);
 		if (!digest) {
 			ldns_rr_free(ds);
 			return NULL;
 		}
 		break;
 	case LDNS_SHA256:
-#ifdef USE_SHA2
-		digest = LDNS_XMALLOC(uint8_t, SHA256_DIGEST_LENGTH);
+		digest = LDNS_XMALLOC(uint8_t, LDNS_SHA256_DIGEST_LENGTH);
 		if (!digest) {
 			ldns_rr_free(ds);
 			return NULL;
 		}
-#else
-		printf("[XX] SHA2 disabled\n");
-		return NULL;
-#endif
-break;
+		break;
 	}
 
 	data_buf = ldns_buffer_new(LDNS_MAX_PACKETLEN);
@@ -520,24 +515,24 @@ break;
 	}
 	switch(h) {
 	case LDNS_SHA1:
-		(void) SHA1((unsigned char *) ldns_buffer_begin(data_buf),
-				  ldns_buffer_position(data_buf),
-				  (unsigned char*) digest);
+		(void) ldns_sha1((unsigned char *) ldns_buffer_begin(data_buf),
+		                 (unsigned int) ldns_buffer_position(data_buf),
+		                 (unsigned char *) digest);
 
-		tmp = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_HEX, SHA_DIGEST_LENGTH,
-							   digest);
+		tmp = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_HEX,
+		                            LDNS_SHA1_DIGEST_LENGTH,
+		                            digest);
 		ldns_rr_push_rdf(ds, tmp);
 
 		break;
 	case LDNS_SHA256:
-#ifdef USE_SHA2
-		(void) SHA256((unsigned char *) ldns_buffer_begin(data_buf),
-				    ldns_buffer_position(data_buf),
-				    (unsigned char*) digest);
-		tmp = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_HEX, SHA256_DIGEST_LENGTH,
-							   digest);
+		(void) ldns_sha256((unsigned char *) ldns_buffer_begin(data_buf),
+		                   (unsigned int) ldns_buffer_position(data_buf),
+		                   (unsigned char *) digest);
+		tmp = ldns_rdf_new_frm_data(LDNS_RDF_TYPE_HEX,
+		                            LDNS_SHA256_DIGEST_LENGTH,
+		                            digest);
 		ldns_rr_push_rdf(ds, tmp);
-#endif
 		break;
 	}
 
@@ -548,8 +543,8 @@ break;
 
 ldns_rdf *
 ldns_dnssec_create_nsec_bitmap(ldns_rr_type rr_type_list[],
-						 size_t size,
-						 ldns_rr_type nsec_type)
+                               size_t size,
+                               ldns_rr_type nsec_type)
 {
 	size_t i;
 	uint8_t *bitmap;
@@ -586,15 +581,6 @@ ldns_dnssec_create_nsec_bitmap(ldns_rr_type rr_type_list[],
 				   (int) (7 - (i_type % 8)),
 				   true);
 	}
-	/* always add nsec (if this is not nsec3 and rrsig */
-	i_type = LDNS_RR_TYPE_RRSIG;
-	ldns_set_bit(bitmap + (int) i_type / 8, (int) (7 - (i_type % 8)), true);
-	i_type = nsec_type;
-	if (i_type != LDNS_RR_TYPE_NSEC3) {
-		ldns_set_bit(bitmap + (int) i_type / 8,
-				   (int) (7 - (i_type % 8)),
-				   true);
-	}
 
 	/* fold it into windows TODO: can this be done directly? */
 	memset(cur_data, 0, 32);
@@ -616,14 +602,13 @@ ldns_dnssec_create_nsec_bitmap(ldns_rr_type rr_type_list[],
 			cur_window++;
 			cur_window_max = 0;
 			memset(cur_data, 0, 32);
-		} else {
-			cur_data[i%32] = bitmap[i];
-			if (bitmap[i] > 0) {
-				cur_window_max = i%32;
-			}
+		}
+		cur_data[i%32] = bitmap[i];
+		if (bitmap[i] > 0) {
+			cur_window_max = i%32;
 		}
 	}
-	if (cur_window_max > 0) {
+	if (cur_window_max > 0 || cur_data[0] != 0) {
 		/* this window has stuff, add it */
 		data = LDNS_XREALLOC(data,
 						 uint8_t,
@@ -644,6 +629,34 @@ ldns_dnssec_create_nsec_bitmap(ldns_rr_type rr_type_list[],
 	return bitmap_rdf;
 }
 
+int
+ldns_dnssec_rrsets_contains_type(ldns_dnssec_rrsets *rrsets,
+                                 ldns_rr_type type)
+{
+	ldns_dnssec_rrsets *cur_rrset = rrsets;
+	while (cur_rrset) {
+		if (cur_rrset->type == type) {
+			return 1;
+		}
+		cur_rrset = cur_rrset->next;
+	}
+	return 0;
+}
+
+/* returns true if the current dnssec_rrset from the given list of rrsets
+ * is glue */
+static int
+is_glue(ldns_dnssec_rrsets *cur_rrsets, ldns_dnssec_rrsets *orig_rrsets)
+{
+	/* only glue if a or aaaa if there are no ns, unless there is soa */
+	return (cur_rrsets->type == LDNS_RR_TYPE_A ||
+	        cur_rrsets->type ==  LDNS_RR_TYPE_AAAA) &&
+	        (ldns_dnssec_rrsets_contains_type(orig_rrsets,
+	              LDNS_RR_TYPE_NS) &&
+	        !ldns_dnssec_rrsets_contains_type(orig_rrsets,
+	              LDNS_RR_TYPE_SOA));
+}
+
 ldns_rr *
 ldns_dnssec_create_nsec(ldns_dnssec_name *from,
                         ldns_dnssec_name *to,
@@ -666,14 +679,22 @@ ldns_dnssec_create_nsec(ldns_dnssec_name *from,
 
 	cur_rrsets = from->rrsets;
 	while (cur_rrsets) {
+		if (is_glue(cur_rrsets, from->rrsets)) {
+			cur_rrsets = cur_rrsets->next;
+			continue;
+		}
 		types[type_count] = cur_rrsets->type;
 		type_count++;
 		cur_rrsets = cur_rrsets->next;
 	}
+	types[type_count] = LDNS_RR_TYPE_RRSIG;
+	type_count++;
+	types[type_count] = LDNS_RR_TYPE_NSEC;
+	type_count++;
 
 	ldns_rr_push_rdf(nsec_rr, ldns_dnssec_create_nsec_bitmap(types,
-												  type_count,
-												  nsec_type));
+	                               type_count,
+	                               nsec_type));
 
 	return nsec_rr;
 }
@@ -702,33 +723,45 @@ ldns_dnssec_create_nsec3(ldns_dnssec_name *from,
 
 	nsec_rr = ldns_rr_new_frm_type(LDNS_RR_TYPE_NSEC3);
 	ldns_rr_set_owner(nsec_rr,
-				   ldns_nsec3_hash_name(ldns_dnssec_name_name(from),
-								    algorithm,
-								    iterations,
-								    salt_length,
-								    salt));
+	                  ldns_nsec3_hash_name(ldns_dnssec_name_name(from),
+	                  algorithm,
+	                  iterations,
+	                  salt_length,
+	                  salt));
 	status = ldns_dname_cat(ldns_rr_owner(nsec_rr), zone_name);
 	ldns_nsec3_add_param_rdfs(nsec_rr,
-						 algorithm,
-						 flags,
-						 iterations,
-						 salt_length,
-						 salt);
+	                          algorithm,
+	                          flags,
+	                          iterations,
+	                          salt_length,
+	                          salt);
 
 	cur_rrsets = from->rrsets;
 	while (cur_rrsets) {
+		if (is_glue(cur_rrsets, from->rrsets)) {
+			cur_rrsets = cur_rrsets->next;
+			continue;
+		}
 		types[type_count] = cur_rrsets->type;
 		type_count++;
 		cur_rrsets = cur_rrsets->next;
 	}
-
+	/* always add rrsig type if this is not an unsigned 
+	 * delegation
+	 */
+	if (type_count > 0 &&
+	    !(type_count == 1 && types[0] == LDNS_RR_TYPE_NS)) {
+		types[type_count] = LDNS_RR_TYPE_RRSIG;
+		type_count++;
+	}
+	
 	/* leave next rdata empty if they weren't precomputed yet */
 	if (to && to->hashed_name) {
-		ldns_rr_set_rdf(nsec_rr,
-					 ldns_rdf_clone(to->hashed_name),
-					 4);
+		(void) ldns_rr_set_rdf(nsec_rr,
+		                       ldns_rdf_clone(to->hashed_name),
+		                       4);
 	} else {
-		ldns_rr_set_rdf(nsec_rr, NULL, 4);
+		(void) ldns_rr_set_rdf(nsec_rr, NULL, 4);
 	}
 
 	ldns_rr_push_rdf(nsec_rr,
@@ -868,11 +901,13 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 	char *orig_owner_str;
 	size_t hashed_owner_str_len;
 	ldns_rdf *hashed_owner;
-	char *hashed_owner_str;
+	unsigned char *hashed_owner_str;
 	char *hashed_owner_b32;
-	int hashed_owner_b32_len;
+	size_t hashed_owner_b32_len;
 	uint32_t cur_it;
-	char *hash = NULL;
+	/* define to contain the largest possible hash, which is
+	 * sha1 at the moment */
+	unsigned char hash[LDNS_SHA1_DIGEST_LENGTH];
 	ldns_status status;
 	
 	/* prepare the owner name according to the draft section bla */
@@ -882,31 +917,30 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 	algorithm = algorithm;
 	
 	hashed_owner_str_len = salt_length + ldns_rdf_size(name);
-	hashed_owner_str = LDNS_XMALLOC(char, hashed_owner_str_len);
+	hashed_owner_str = LDNS_XMALLOC(unsigned char, hashed_owner_str_len);
 	memcpy(hashed_owner_str, ldns_rdf_data(name), ldns_rdf_size(name));
 	memcpy(hashed_owner_str + ldns_rdf_size(name), salt, salt_length);
 
 	for (cur_it = iterations + 1; cur_it > 0; cur_it--) {
-		hash = (char *) SHA1((unsigned char *) hashed_owner_str,
-						 hashed_owner_str_len,
-						 NULL);
+		(void) ldns_sha1((unsigned char *) hashed_owner_str,
+		                 (unsigned int) hashed_owner_str_len, hash);
 
 		LDNS_FREE(hashed_owner_str);
-		hashed_owner_str_len = salt_length + SHA_DIGEST_LENGTH;
-		hashed_owner_str = LDNS_XMALLOC(char, hashed_owner_str_len);
+		hashed_owner_str_len = salt_length + LDNS_SHA1_DIGEST_LENGTH;
+		hashed_owner_str = LDNS_XMALLOC(unsigned char, hashed_owner_str_len);
 		if (!hashed_owner_str) {
 			fprintf(stderr, "Memory error\n");
 			abort();
 		}
-		memcpy(hashed_owner_str, hash, SHA_DIGEST_LENGTH);
-		memcpy(hashed_owner_str + SHA_DIGEST_LENGTH, salt, salt_length);
-		hashed_owner_str_len = SHA_DIGEST_LENGTH + salt_length;
+		memcpy(hashed_owner_str, hash, LDNS_SHA1_DIGEST_LENGTH);
+		memcpy(hashed_owner_str + LDNS_SHA1_DIGEST_LENGTH, salt, salt_length);
+		hashed_owner_str_len = LDNS_SHA1_DIGEST_LENGTH + salt_length;
 	}
 
 	LDNS_FREE(orig_owner_str);
 	LDNS_FREE(hashed_owner_str);
 	hashed_owner_str = hash;
-	hashed_owner_str_len = SHA_DIGEST_LENGTH;
+	hashed_owner_str_len = LDNS_SHA1_DIGEST_LENGTH;
 
 	hashed_owner_b32 = LDNS_XMALLOC(char,
 							  ldns_b32_ntop_calculate_size(
@@ -921,8 +955,10 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 		fprintf(stderr, "Error in base32 extended hex encoding ");
 		fprintf(stderr, "of hashed owner name (name: ");
 		ldns_rdf_print(stderr, name);
-		fprintf(stderr, ", return code: %d)\n", hashed_owner_b32_len);
-		exit(4);
+		fprintf(stderr, ", return code: %u)\n",
+		        (unsigned int) hashed_owner_b32_len);
+		LDNS_FREE(hashed_owner_b32);
+		return NULL;
 	}
 	hashed_owner_str_len = hashed_owner_b32_len;
 	hashed_owner_b32[hashed_owner_b32_len] = '\0';
@@ -930,7 +966,8 @@ ldns_nsec3_hash_name(ldns_rdf *name,
 	status = ldns_str2rdf_dname(&hashed_owner, hashed_owner_b32);
 	if (status != LDNS_STATUS_OK) {
 		fprintf(stderr, "Error creating rdf from %s\n", hashed_owner_b32);
-		exit(1);
+		LDNS_FREE(hashed_owner_b32);
+		return NULL;
 	}
 
 	LDNS_FREE(hashed_owner_b32);
@@ -947,21 +984,25 @@ ldns_nsec3_add_param_rdfs(ldns_rr *rr,
 {
 	ldns_rdf *salt_rdf = NULL;
 	uint8_t *salt_data = NULL;
+	ldns_rdf *old;
+
+	old = ldns_rr_set_rdf(rr,
+	                      ldns_rdf_new_frm_data(LDNS_RDF_TYPE_INT8,
+	                                            1, (void*)&algorithm),
+	                      0);
+	if (old) ldns_rdf_deep_free(old);
+
+	old = ldns_rr_set_rdf(rr,
+	                      ldns_rdf_new_frm_data(LDNS_RDF_TYPE_INT8,
+	                                            1, (void*)&flags),
+	                      1);
+	if (old) ldns_rdf_deep_free(old);
 	
-	ldns_rr_set_rdf(rr,
-				 ldns_rdf_new_frm_data(LDNS_RDF_TYPE_INT8,
-								   1,
-								   (void*)&algorithm),
-				 0);
-	ldns_rr_set_rdf(rr,
-				 ldns_rdf_new_frm_data(LDNS_RDF_TYPE_INT8,
-								   1,
-								   (void*)&flags),
-				 1);
-	ldns_rr_set_rdf(rr,
-				 ldns_native2rdf_int16(LDNS_RDF_TYPE_INT16,
-								   iterations),
-				 2);
+	old = ldns_rr_set_rdf(rr,
+                          ldns_native2rdf_int16(LDNS_RDF_TYPE_INT16,
+                                                iterations),
+	                      2);
+	if (old) ldns_rdf_deep_free(old);
 	
 	salt_data = LDNS_XMALLOC(uint8_t, salt_length + 1);
 	salt_data[0] = salt_length;
@@ -970,10 +1011,29 @@ ldns_nsec3_add_param_rdfs(ldns_rr *rr,
 							   salt_length + 1,
 							   salt_data);
 
-	ldns_rr_set_rdf(rr, salt_rdf, 3);
+	old = ldns_rr_set_rdf(rr, salt_rdf, 3);
+	if (old) ldns_rdf_deep_free(old);
 	LDNS_FREE(salt_data);
 }
 
+static int
+rr_list_delegation_only(ldns_rdf *origin, ldns_rr_list *rr_list)
+{
+	size_t i;
+	ldns_rr *cur_rr;
+	if (!origin || !rr_list) return 0;
+	for (i = 0; i < ldns_rr_list_rr_count(rr_list); i++) {
+		cur_rr = ldns_rr_list_rr(rr_list, i);
+		if (ldns_dname_compare(ldns_rr_owner(cur_rr), origin) == 0) {
+			return 0;
+		}
+		if (ldns_rr_get_type(cur_rr) != LDNS_RR_TYPE_NS) {
+			return 0;
+		}
+	}
+	return 1;
+}
+
 /* this will NOT return the NSEC3  completed, you will have to run the
    finalize function on the rrlist later! */
 ldns_rr *
@@ -1023,7 +1083,7 @@ ldns_create_nsec3(ldns_rdf *cur_owner,
 						 iterations,
 						 salt_length,
 						 salt);
-	ldns_rr_set_rdf(nsec, NULL, 4);
+	(void) ldns_rr_set_rdf(nsec, NULL, 4);
 
 	bitmap[0] = 0;
 	for (i = 0; i < ldns_rr_list_rr_count(rrs); i++) {
@@ -1045,8 +1105,9 @@ ldns_create_nsec3(ldns_rdf *cur_owner,
 					   true);
 		}
 	}
-	/* add NSEC and RRSIG anyway */
-	if (!emptynonterminal) {
+	/* add NSEC and RRSIG anyway, but only if this is not an ENT or
+	 * an unsigned delegation */
+	if (!emptynonterminal && !rr_list_delegation_only(cur_zone, rrs)) {
 		i_type = LDNS_RR_TYPE_RRSIG;
 		if (i_type / 8 > bm_len) {
 			bitmap = LDNS_XREALLOC(bitmap, uint8_t, (i_type / 8) + 1);
@@ -1101,16 +1162,13 @@ ldns_create_nsec3(ldns_rdf *cur_owner,
 			}
 		}
 	}
-	if (cur_window_max > 0) {
-		/* this window has stuff, add it */
-		data = LDNS_XREALLOC(data,
-						 uint8_t,
-						 cur_data_size + cur_window_max + 3);
-		data[cur_data_size] = cur_window;
-		data[cur_data_size + 1] = cur_window_max + 1;
-		memcpy(data + cur_data_size + 2, cur_data, cur_window_max+1);
-		cur_data_size += cur_window_max + 3;
-	}
+	data = LDNS_XREALLOC(data,
+					 uint8_t,
+					 cur_data_size + cur_window_max + 3);
+	data[cur_data_size] = cur_window;
+	data[cur_data_size + 1] = cur_window_max + 1;
+	memcpy(data + cur_data_size + 2, cur_data, cur_window_max+1);
+	cur_data_size += cur_window_max + 3;
 
 	ldns_rr_push_rdf(nsec,
 	                 ldns_rdf_new_frm_data(LDNS_RDF_TYPE_NSEC,
@@ -1140,7 +1198,7 @@ ldns_nsec3_flags(const ldns_rr *nsec3_rr)
 	if (nsec3_rr && ldns_rr_get_type(nsec3_rr) == LDNS_RR_TYPE_NSEC3 &&
 	    ldns_rdf_size(ldns_rr_rdf(nsec3_rr, 1)) > 0
 	    ) {
-		return ldns_rdf2native_int8(ldns_rr_rdf(nsec3_rr, 0));
+		return ldns_rdf2native_int8(ldns_rr_rdf(nsec3_rr, 1));
 	}
 	return 0;
 }
@@ -1315,6 +1373,7 @@ ldns_nsec_covers_name(const ldns_rr *nsec, const ldns_rdf *name)
 	return result;
 }
 
+#ifdef HAVE_SSL
 /* sig may be null - if so look in the packet */
 ldns_status
 ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o, 
@@ -1354,8 +1413,9 @@ ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o,
 	 * sigs for the type t
 	 */
 	t_netorder = htons(t); /* rdf are in network order! */
+	/* a type identifier is a 16-bit number, so the size is 2 bytes */
 	rdf_t = ldns_rdf_new(LDNS_RDF_TYPE_TYPE,
-					 sizeof(ldns_rr_type),
+					 2,
 					 &t_netorder);
 	sigs_covered = ldns_rr_list_subtype_by_rdf(sigs, rdf_t, 0);
 	
@@ -1374,91 +1434,7 @@ ldns_pkt_verify(ldns_pkt *p, ldns_rr_type t, ldns_rdf *o,
 
 	return ldns_verify(rrset, sigs, k, good_keys);
 }
-
-#if 0
-ldns_rr_list *
-ldns_zone_create_nsecs(const ldns_zone *zone, ldns_rr_list *orig_zone_rrs, ldns_rr_list *glue_rrs)
-{
-	ldns_rr_list *nsec_rrs = ldns_rr_list_new();
-	ldns_rdf *start_dname = NULL;
-	ldns_rdf *next_dname = NULL;
-	ldns_rdf *cur_dname = NULL;
-
-	ldns_rr *nsec = NULL;
-	ldns_rr *next_rr = NULL;
-	size_t i;
-
-	/* add nsecs */
-	for (i = 0; i < ldns_rr_list_rr_count(orig_zone_rrs); i++) {
-		if (!start_dname) {
-			/*start_dname = ldns_rr_owner(ldns_zone_soa(zone));*/
-			start_dname = ldns_rr_owner(ldns_rr_list_rr(orig_zone_rrs, i));
-			cur_dname = start_dname;
-		} else {
-			next_rr = ldns_rr_list_rr(orig_zone_rrs, i);
-			next_dname = ldns_rr_owner(next_rr);
-			if (ldns_rdf_compare(cur_dname, next_dname) != 0) {
-				/* skip glue */
-				if (ldns_rr_list_contains_rr(glue_rrs, next_rr)) {
-					cur_dname = next_dname;
-				} else {
-					nsec = ldns_create_nsec(cur_dname, 
-									    next_dname,
-									    orig_zone_rrs);
-					ldns_rr_set_ttl(nsec,
-								 ldns_rdf2native_int32(
-									ldns_rr_rdf(ldns_zone_soa(zone),
-											  6)));
-					ldns_rr_list_push_rr(nsec_rrs, nsec);
-					/*start_dname = next_dname;*/
-					cur_dname = next_dname;
-				}
-			}
-		}
-	}
-	nsec = ldns_create_nsec(cur_dname, 
-					    start_dname,
-					    orig_zone_rrs);
-	ldns_rr_list_push_rr(nsec_rrs, nsec);
-	ldns_rr_set_ttl(nsec, 
-				 ldns_rdf2native_int32(ldns_rr_rdf(ldns_zone_soa(zone),
-											6)));
-
-	return nsec_rrs;
-}
-
-
-/* return a clone of the given list without RRSIGS and NSEC(3)'s */
-/* if removed_rrs is not null, push clones of sigs and nsecs there */
-ldns_rr_list *
-ldns_rr_list_strip_dnssec(ldns_rr_list *rr_list, ldns_rr_list *removed_rrs)
-{
-	size_t i;
-	ldns_rr_list *new_list;
-	ldns_rr *cur_rr;
-	
-	if (!rr_list) {
-		return NULL;
-	}
-
-	new_list = ldns_rr_list_new();
-
-	for (i = 0; i < ldns_rr_list_rr_count(rr_list); i++) {
-		cur_rr = ldns_rr_list_rr(rr_list, i);
-		if (ldns_rr_get_type(cur_rr) != LDNS_RR_TYPE_RRSIG &&
-		    ldns_rr_get_type(cur_rr) != LDNS_RR_TYPE_NSEC &&
-		    ldns_rr_get_type(cur_rr) != LDNS_RR_TYPE_NSEC3) {
-			ldns_rr_list_push_rr(new_list, ldns_rr_clone(cur_rr));
-		} else {
-			if (removed_rrs) {
-				ldns_rr_list_push_rr(removed_rrs, ldns_rr_clone(cur_rr));
-			}
-		}
-	}
-
-	return new_list;
-}
-#endif
+#endif /* HAVE_SSL */
 
 ldns_status
 ldns_dnssec_chain_nsec3_list(ldns_rr_list *nsec3_rrs)
@@ -1571,13 +1547,15 @@ ldns_dnssec_default_replace_signatures(ldns_rr *sig, void *n)
 	return LDNS_SIGNATURE_REMOVE_ADD_NEW;
 }
 
+#ifdef HAVE_SSL
 ldns_rdf *
 ldns_convert_dsa_rrsig_asn12rdf(const ldns_buffer *sig,
 						  const long sig_len)
 {
 	ldns_rdf *sigdata_rdf;
 	DSA_SIG *dsasig;
-	unsigned char *dsasig_data = ldns_buffer_begin(sig);
+	unsigned char *dsasig_data = (unsigned char*)ldns_buffer_begin(sig);
+	size_t byte_offset;
 
 	dsasig = d2i_DSA_SIG(NULL,
 					 (const unsigned char **)&dsasig_data,
@@ -1588,8 +1566,18 @@ ldns_convert_dsa_rrsig_asn12rdf(const ldns_buffer *sig,
 
 	dsasig_data = LDNS_XMALLOC(unsigned char, 41);
 	dsasig_data[0] = 0;
-	BN_bn2bin(dsasig->r, &dsasig_data[1]);
-	BN_bn2bin(dsasig->s, &dsasig_data[21]);
+	byte_offset = (size_t) (20 - BN_num_bytes(dsasig->r));
+	if (byte_offset > 20) {
+		return NULL;
+	}
+	memset(&dsasig_data[1], 0, byte_offset);
+	BN_bn2bin(dsasig->r, &dsasig_data[1 + byte_offset]);
+	byte_offset = (size_t) (20 - BN_num_bytes(dsasig->s));
+	if (byte_offset > 20) {
+		return NULL;
+	}
+	memset(&dsasig_data[21], 0, byte_offset);
+	BN_bn2bin(dsasig->s, &dsasig_data[21 + byte_offset]);
 	
 	sigdata_rdf = ldns_rdf_new(LDNS_RDF_TYPE_B64, 41, dsasig_data);
 	DSA_SIG_free(dsasig);
@@ -1611,9 +1599,11 @@ ldns_convert_dsa_rrsig_rdf2asn1(ldns_buffer *target_buffer,
 	/* extract the R and S field from the sig buffer */
 	t = ldns_rdf_data(sig_rdf)[0];
 	R = BN_new();
-	(void) BN_bin2bn(ldns_rdf_data(sig_rdf) + 1, SHA_DIGEST_LENGTH, R);
+	(void) BN_bin2bn((unsigned char *) ldns_rdf_data(sig_rdf) + 1,
+	                 SHA_DIGEST_LENGTH, R);
 	S = BN_new();
-	(void) BN_bin2bn(ldns_rdf_data(sig_rdf) + 21, SHA_DIGEST_LENGTH, S);
+	(void) BN_bin2bn((unsigned char *) ldns_rdf_data(sig_rdf) + 21,
+	                 SHA_DIGEST_LENGTH, S);
 
 	dsasig = DSA_SIG_new();
 	if (!dsasig) {
@@ -1624,9 +1614,13 @@ ldns_convert_dsa_rrsig_rdf2asn1(ldns_buffer *target_buffer,
 	dsasig->s = S;
 	
 	raw_sig_len = i2d_DSA_SIG(dsasig, &raw_sig);
-	
-	if (ldns_buffer_reserve(target_buffer, raw_sig_len)) {
-		ldns_buffer_write(target_buffer, raw_sig, raw_sig_len);
+	if (raw_sig_len < 0) {
+		DSA_SIG_free(dsasig);
+		free(raw_sig);
+		return LDNS_STATUS_SSL_ERR;
+	}
+	if (ldns_buffer_reserve(target_buffer, (size_t) raw_sig_len)) {
+		ldns_buffer_write(target_buffer, raw_sig, (size_t)raw_sig_len);
 	}
 
 	DSA_SIG_free(dsasig);
@@ -1634,7 +1628,4 @@ ldns_convert_dsa_rrsig_rdf2asn1(ldns_buffer *target_buffer,
 
 	return ldns_buffer_status(target_buffer);
 }
-
-
-
 #endif /* HAVE_SSL */
