@@ -17,6 +17,7 @@
 #include <openssl/rand.h>
 #include <openssl/err.h>
 #include <openssl/md5.h>
+#endif /* HAVE_SSL */
 
 ldns_rr *
 ldns_create_empty_rrsig(ldns_rr_list *rrset,
@@ -105,7 +106,7 @@ ldns_create_empty_rrsig(ldns_rr_list *rrset,
 	return current_sig;
 }
 
-
+#ifdef HAVE_SSL
 ldns_rdf *
 ldns_sign_public_buffer(ldns_buffer *sign_buf, ldns_key *current_key)
 {
@@ -128,14 +129,12 @@ ldns_sign_public_buffer(ldns_buffer *sign_buf, ldns_key *current_key)
 		break;
 #ifdef USE_SHA2
 	case LDNS_SIGN_RSASHA256:
-	case LDNS_SIGN_RSASHA256_NSEC3:
 		b64rdf = ldns_sign_public_evp(
 				   sign_buf,
 				   ldns_key_evp_key(current_key),
 				   EVP_sha256());
 		break;
 	case LDNS_SIGN_RSASHA512:
-	case LDNS_SIGN_RSASHA512_NSEC3:
 		b64rdf = ldns_sign_public_evp(
 				   sign_buf,
 				   ldns_key_evp_key(current_key),
@@ -385,7 +384,6 @@ ldns_sign_public_evp(ldns_buffer *to_sign,
 	return sigdata_rdf;
 }
 
-
 ldns_rdf *
 ldns_sign_public_rsasha1(ldns_buffer *to_sign, RSA *key)
 {
@@ -454,6 +452,23 @@ ldns_sign_public_rsamd5(ldns_buffer *to_sign, RSA *key)
 	ldns_buffer_free(b64sig);
 	return sigdata_rdf;
 }
+#endif /* HAVE_SSL */
+
+static int
+ldns_dnssec_name_has_only_a(ldns_dnssec_name *cur_name)
+{
+	ldns_dnssec_rrsets *cur_rrset;
+	cur_rrset = cur_name->rrsets;
+	while (cur_rrset) {
+		if (cur_rrset->type != LDNS_RR_TYPE_A &&
+			cur_rrset->type != LDNS_RR_TYPE_AAAA) {
+			return 0;
+		} else {
+			cur_rrset = cur_rrset->next;
+		}
+	}
+	return 1;
+}
 
 ldns_status
 ldns_dnssec_zone_mark_glue(ldns_dnssec_zone *zone)
@@ -466,10 +481,7 @@ ldns_dnssec_zone_mark_glue(ldns_dnssec_zone *zone)
 	while (cur_node != LDNS_RBTREE_NULL) {
 		cur_name = (ldns_dnssec_name *) cur_node->data;
 		cur_node = ldns_rbtree_next(cur_node);
-		if (cur_name->rrsets && 
-		    (cur_name->rrsets->type == LDNS_RR_TYPE_A ||
-			cur_name->rrsets->type == LDNS_RR_TYPE_AAAA
-			)) {
+		if (ldns_dnssec_name_has_only_a(cur_name)) {
 			/* assume glue XXX check for zone cur */
 			cur_owner = ldns_rdf_clone(ldns_rr_owner(
 					      cur_name->rrsets->rrs->rr));
@@ -589,6 +601,7 @@ ldns_dnssec_zone_create_nsecs(ldns_dnssec_zone *zone,
 	return LDNS_STATUS_OK;
 }
 
+#ifdef HAVE_SSL
 ldns_status
 ldns_dnssec_zone_create_nsec3s(ldns_dnssec_zone *zone,
 						 ldns_rr_list *new_rrs,
@@ -666,6 +679,7 @@ ldns_dnssec_zone_create_nsec3s(ldns_dnssec_zone *zone,
 	ldns_rr_list_free(nsec3_list);
 	return result;
 }
+#endif /* HAVE_SSL */
 
 ldns_dnssec_rrs *
 ldns_dnssec_remove_signatures(ldns_dnssec_rrs *signatures,
@@ -755,6 +769,7 @@ ldns_dnssec_remove_signatures(ldns_dnssec_rrs *signatures,
 	return base_rrs;
 }
 
+#ifdef HAVE_SSL
 ldns_status
 ldns_dnssec_zone_create_rrsigs(ldns_dnssec_zone *zone,
                                ldns_rr_list *new_rrs,
@@ -814,9 +829,18 @@ ldns_dnssec_zone_create_rrsigs(ldns_dnssec_zone *zone,
 				
 				/* only sign non-delegation RRsets */
 				/* (glue should have been marked earlier) */
-				if (ldns_rr_list_type(rr_list) != LDNS_RR_TYPE_NS ||
+				if ((ldns_rr_list_type(rr_list) != LDNS_RR_TYPE_NS ||
 					ldns_dname_compare(ldns_rr_list_owner(rr_list),
-					zone->soa->name) == 0) {
+					zone->soa->name) == 0) &&
+					/* OK, there is also the possibility that the record
+					 * is glue, but at the same owner name as other records that
+					 * are not NS nor A/AAAA. Bleh, our current data structure
+					 * doesn't really support that... */
+					!((ldns_rr_list_type(rr_list) == LDNS_RR_TYPE_A ||
+					 ldns_rr_list_type(rr_list) == LDNS_RR_TYPE_AAAA) &&
+					 !ldns_dname_compare(ldns_rr_list_owner(rr_list), zone->soa->name) == 0 &&
+					 ldns_dnssec_zone_find_rrset(zone, ldns_rr_list_owner(rr_list), LDNS_RR_TYPE_NS)
+					 )) {
 
 					siglist = ldns_sign_public(rr_list, key_list);
 					for (i = 0; i < ldns_rr_list_rr_count(siglist); i++) {
@@ -935,7 +959,7 @@ ldns_dnssec_zone_sign_nsec3(ldns_dnssec_zone *zone,
 
 		nsec3 = ((ldns_dnssec_name *)zone->names->root->data)->nsec;
 		if (nsec3 && ldns_rr_get_type(nsec3) == LDNS_RR_TYPE_NSEC3) {
-			// no need to recreate
+			/* no need to recreate */
 		} else {
 			if (!ldns_dnssec_zone_find_rrset(zone,
 									   zone->soa->name,
@@ -1066,5 +1090,5 @@ ldns_zone_sign_nsec3(ldns_zone *zone, ldns_key_list *key_list, uint8_t algorithm
 
 	return signed_zone;
 }
+#endif /* HAVE_SSL */
 
-#endif
