@@ -340,6 +340,11 @@ ldns_dnssec_name_free_internal(ldns_dnssec_name *name,
 		if (name->nsec_signatures) {
 			ldns_dnssec_rrs_free_internal(name->nsec_signatures, deep);
 		}
+		if (name->hashed_name) {
+			if (deep) {
+				ldns_rdf_deep_free(name->hashed_name);
+			}
+		}
 		LDNS_FREE(name);
 	}
 }
@@ -409,7 +414,6 @@ ldns_dnssec_name_cmp(const void *a, const void *b)
 	}
 }
 
-
 ldns_status
 ldns_dnssec_name_add_rr(ldns_dnssec_name *name,
 				    ldns_rr *rr)
@@ -432,6 +436,7 @@ ldns_dnssec_name_add_rr(ldns_dnssec_name *name,
 		typecovered = ldns_rdf2rr_type(ldns_rr_rrsig_typecovered(rr));
 	}
 
+#ifdef HAVE_SSL
 	if (rr_type == LDNS_RR_TYPE_NSEC3 ||
 	    typecovered == LDNS_RR_TYPE_NSEC3) {
 		name_name = ldns_nsec3_hash_name_frm_nsec3(rr,
@@ -440,6 +445,9 @@ ldns_dnssec_name_add_rr(ldns_dnssec_name *name,
 	} else {
 		name_name = ldns_dnssec_name_name(name);
 	}
+#else
+	name_name = ldns_dnssec_name_name(name);
+#endif /* HAVE_SSL */
 
 	if (rr_type == LDNS_RR_TYPE_NSEC ||
 	    rr_type == LDNS_RR_TYPE_NSEC3) {
@@ -603,10 +611,10 @@ ldns_dname_compare_v(const void *a, const void *b) {
 	return ldns_dname_compare((ldns_rdf *)a, (ldns_rdf *)b);
 }
 
-
+#ifdef HAVE_SSL
 ldns_rbnode_t *
 ldns_dnssec_zone_find_nsec3_original(ldns_dnssec_zone *zone,
-							  ldns_rr *rr) {
+                                     ldns_rr *rr) {
 	ldns_rbnode_t *current_node = ldns_rbtree_first(zone->names);
 	ldns_dnssec_name *current_name;
 	ldns_rdf *hashed_name;
@@ -615,7 +623,6 @@ ldns_dnssec_zone_find_nsec3_original(ldns_dnssec_zone *zone,
 
 	while (current_node != LDNS_RBTREE_NULL) {
 		current_name = (ldns_dnssec_name *) current_node->data;
-
 		if (!current_name->hashed_name) {
 			current_name->hashed_name =
 				ldns_nsec3_hash_name_frm_nsec3(rr, current_name->name);
@@ -623,10 +630,12 @@ ldns_dnssec_zone_find_nsec3_original(ldns_dnssec_zone *zone,
 		if (ldns_dname_compare(hashed_name,
 						   current_name->hashed_name)
 		    == 0) {
+			ldns_rdf_deep_free(hashed_name);
 			return current_node;
 		}
 		current_node = ldns_rbtree_next(current_node);
 	}
+	ldns_rdf_deep_free(hashed_name);
 	return NULL;
 }
 
@@ -686,6 +695,7 @@ ldns_dnssec_zone_add_rr(ldns_dnssec_zone *zone, ldns_rr *rr)
 
 	return result;
 }
+#endif /* HAVE_SSL */
 
 void
 ldns_dnssec_zone_names_print(FILE *out, ldns_rbtree_t *tree, bool print_soa)
