@@ -29,8 +29,10 @@ usage(FILE *stream, const char *progname)
 	fprintf(stream, "\n\targuments may be placed in random order\n");
 	fprintf(stream, "\n  Options:\n");
 	fprintf(stream, "\t-D\t\tenable DNSSEC (DO bit)\n");
+#ifdef HAVE_SSL
 	fprintf(stream, "\t-T\t\ttrace from the root down to <name>\n");
 	fprintf(stream, "\t-S\t\tchase signature(s) from <name> to a know key [*]\n");
+#endif /*HAVE_SSL*/
 	fprintf(stream, "\t-V <number>\tverbosity (0-5)\n");
 	fprintf(stream, "\t-Q\t\tquiet mode (overrules -V)\n");
 	fprintf(stream, "\n");
@@ -43,16 +45,13 @@ usage(FILE *stream, const char *progname)
 	fprintf(stream, "\n  Query options:\n");
 	fprintf(stream, "\t-4\t\tstay on ip4\n");
 	fprintf(stream, "\t-6\t\tstay on ip6\n");
-	fprintf(stream, "\t-a\t\tonly query the first nameserver (default is to try all)\n");
+	fprintf(stream, "\t-a\t\tfallback to EDNS0 and TCP if the answer is truncated\n");
 	fprintf(stream, "\t-b <bufsize>\tuse <bufsize> as the buffer size (defaults to 512 b)\n");
 	fprintf(stream, "\t-c <file>\t\tuse file for rescursive nameserver configuration (/etc/resolv.conf)\n");
 	fprintf(stream, "\t-k <file>\tspecify a file that contains a trusted DNSSEC key [**]\n");
 	fprintf(stream, "\t\t\tused to verify any signatures in the current answer\n");
 	fprintf(stream, "\t-o <mnemonic>\tset flags to: [QR|qr][AA|aa][TC|tc][RD|rd][CD|cd][RA|ra][AD|ad]\n");
 	fprintf(stream, "\t\t\tlowercase: unset bit, uppercase: set bit\n");
-#if 0
-	fprintf(stream, "\t-O <opcode>\tset the opcode to: [query, iquery, status, notify, update]]\n");
-#endif
 	fprintf(stream, "\t-p <port>\tuse <port> as remote port number\n");
 	fprintf(stream, "\t-s\t\tshow the DS RR for each key in a packet\n");
 	fprintf(stream, "\t-u\t\tsend the query with udp (the default)\n");
@@ -61,7 +60,7 @@ usage(FILE *stream, const char *progname)
 	fprintf(stream, "\t-r <file>\t\tuse file as root servers hint file\n");
 	fprintf(stream, "\t-t\t\tsend the query with tcp (connected)\n");
 	fprintf(stream, "\t-d <domain>\t\tuse domain as the start point for the trace\n");
-        fprintf(stream, "\t-y <name:key[:algo]>\tspecify named base64 tsig key, and optional an\n\t\t\talgorithm (defaults to hmac-md5.sig-alg.reg.int)\n");
+    fprintf(stream, "\t-y <name:key[:algo]>\tspecify named base64 tsig key, and optional an\n\t\t\talgorithm (defaults to hmac-md5.sig-alg.reg.int)\n");
 	fprintf(stream, "\t-z\t\tdon't randomize the nameservers before use\n");
 	fprintf(stream, "\n  [*] = enables/implies DNSSEC\n");
 	fprintf(stream, "  [**] = can be given more than once\n");
@@ -117,11 +116,11 @@ main(int argc, char *argv[])
 	char		*tsig_name = NULL;
 	char		*tsig_data = NULL;
 	char 		*tsig_algorithm = NULL;
-	ldns_rr		*dnssec_key;
 	size_t		tsig_separator;
 	size_t		tsig_separator2;
 	ldns_rr		*axfr_rr;
 	ldns_status	status;
+	char *type_str;
 	
 	/* list of keys used in dnssec operations */
 	ldns_rr_list	*key_list = ldns_rr_list_new(); 
@@ -134,7 +133,7 @@ main(int argc, char *argv[])
 	uint16_t	qport;
 	uint8_t		qfamily;
 	bool		qdnssec;
-	bool		qfail;
+	bool		qfallback;
 	bool		qds;
 	bool		qusevc;
 	bool 		qrandom;
@@ -144,11 +143,26 @@ main(int argc, char *argv[])
 	ldns_rdf *trace_start_name = NULL;
 
 	int		result = 0;
-	
+
+#ifdef USE_WINSOCK
+	int r;
+	WSADATA wsa_data;
+#endif
+
 	int_type = -1; serv = NULL; type = 0; 
 	int_clas = -1; name = NULL; clas = 0;
 	qname = NULL; 
 	progname = strdup(argv[0]);
+
+#ifdef USE_WINSOCK
+	r = WSAStartup(MAKEWORD(2,2), &wsa_data);
+	if(r != 0) {
+		printf("Failed WSAStartup: %d\n", r);
+		result = EXIT_FAILURE;
+		goto exit;
+	}
+#endif /* USE_WINSOCK */
+		
 	
 	PURPOSE = DRILL_QUERY;
 	qflags = LDNS_RD;
@@ -156,7 +170,7 @@ main(int argc, char *argv[])
 	verbosity = 2;
 	qdnssec = false;
 	qfamily = LDNS_RESOLV_INETANY;
-	qfail = false;
+	qfallback = false;
 	qds = false;
 	qbuf = 0;
 	qusevc = false;
@@ -190,6 +204,7 @@ main(int argc, char *argv[])
 			case 'I':
 				/* reserved for backward compatibility */
 				break;
+#ifdef HAVE_SSL
 			case 'T':
 				if (PURPOSE == DRILL_CHASE) {
 					fprintf(stderr, "-T and -S cannot be used at the same time.\n");
@@ -204,6 +219,7 @@ main(int argc, char *argv[])
 				}
 				PURPOSE = DRILL_CHASE;
 				break;
+#endif /* HAVE_SSL */
 			case 'V':
 				verbosity = atoi(optarg);
 				break;
@@ -237,7 +253,7 @@ main(int argc, char *argv[])
 				break;
 			/* query options */
 			case 'a':
-				qfail = true;
+				qfallback = true;
 				break;
 			case 'b':
 				qbuf = (uint16_t)atoi(optarg);
@@ -252,11 +268,10 @@ main(int argc, char *argv[])
 				qusevc = true;
 				break;
 			case 'k':
-				dnssec_key = read_key_file(optarg);
-				if (!dnssec_key) {
-					error("Could not parse the key file: %s", optarg);
+				status = read_key_file(optarg, key_list);
+				if (status != LDNS_STATUS_OK) {
+					error("Could not parse the key file %s: %s", optarg, ldns_get_errorstr_by_id(status));
 				}
-				ldns_rr_list_push_rr(key_list, dnssec_key);
 				qdnssec = true; /* enable that too */
 				break;
 			case 'o':
@@ -304,30 +319,6 @@ main(int argc, char *argv[])
 					DRILL_OFF(qflags, LDNS_AD);
 				}
 				break;
-#if 0
-			case 'O':
-				if (strstr(optarg, "query")) {
-					opcode = LDNS_PACKET_QUERY;
-					break;
-				}
-				if (strstr(optarg, "iquery")) {
-					opcode = LDNS_PACKET_IQUERY;
-					break;
-				}
-				if (strstr(optarg, "status")) {
-					opcode = LDNS_PACKET_STATUS;
-					break;
-				}
-				if (strstr(optarg, "notify")) {
-					opcode = LDNS_PACKET_NOTIFY;
-					break;
-				}
-				if (strstr(optarg, "update")) {
-					opcode = LDNS_PACKET_UPDATE;
-					break;
-				}
-				break;
-#endif
 			case 'p':
 				qport = (uint16_t)atoi(optarg);
 				if (qport == 0) {
@@ -465,7 +456,8 @@ main(int argc, char *argv[])
 		/* no server given make a resolver from /etc/resolv.conf */
 		status = ldns_resolver_new_frm_file(&res, resolv_conf_file);
 		if (status != LDNS_STATUS_OK) {
-			warning("Could not create a resolver structure: %s", ldns_get_errorstr_by_id(status));
+			warning("Could not create a resolver structure: %s",
+				   ldns_get_errorstr_by_id(status));
 			result = EXIT_FAILURE;
 			goto exit;
 		}
@@ -487,7 +479,7 @@ main(int argc, char *argv[])
 			}
 			ldns_resolver_set_dnssec(cmdline_res, qdnssec);
 			ldns_resolver_set_ip6(cmdline_res, qfamily);
-			ldns_resolver_set_fail(cmdline_res, qfail);
+			ldns_resolver_set_fallback(cmdline_res, qfallback);
 			ldns_resolver_set_usevc(cmdline_res, qusevc);
 
 			cmdline_dname = ldns_dname_new_frm_str(serv);
@@ -527,7 +519,7 @@ main(int argc, char *argv[])
 	ldns_resolver_set_dnssec(res, qdnssec);
 /*	ldns_resolver_set_dnssec_cd(res, qdnssec);*/
 	ldns_resolver_set_ip6(res, qfamily);
-	ldns_resolver_set_fail(res, qfail);
+	ldns_resolver_set_fallback(res, qfallback);
 	ldns_resolver_set_usevc(res, qusevc);
 	ldns_resolver_set_random(res, qrandom);
 	if (qbuf != 0) {
@@ -559,7 +551,7 @@ main(int argc, char *argv[])
 			}
 			qname = ldns_dname_new_frm_str(name);
 			if (!qname) {
-				error("%s", "making qname");
+				error("%s", "parsing query name");
 			}
 			/* don't care about return packet */
 			(void)do_trace(res, qname, type, clas);
@@ -575,7 +567,9 @@ main(int argc, char *argv[])
 				error("%s", "making qname");
 			}
 			/* don't care about return packet */
+#ifdef HAVE_SSL
 			result = do_secure_trace(res, qname, type, clas, key_list, trace_start_name);
+#endif /* HAVE_SSL */
 			clear_root();
 			break;
 		case DRILL_CHASE:
@@ -601,6 +595,7 @@ main(int argc, char *argv[])
 				if (!ldns_pkt_answer(pkt)) {
 					mesg("No answer in packet");
 				} else {
+#ifdef HAVE_SSL
 					ldns_resolver_set_dnssec_anchors(res, ldns_rr_list_clone(key_list));
 					result = do_chase(res, qname, type,
 					                  clas, key_list, 
@@ -616,6 +611,7 @@ main(int argc, char *argv[])
 							mesg("Chase failed.");
 						}
 					}
+#endif /* HAVE_SSL */
 				}
 				ldns_pkt_free(pkt);
 			}
@@ -649,6 +645,9 @@ main(int argc, char *argv[])
 		case DRILL_REVERSE:
 			/* ipv4 or ipv6 addr? */
 			if (strchr(name, ':')) {
+				if (strchr(name, '.')) {
+					error("Syntax error: both '.' and ':' seen in address\n");
+				}
 				name2 = malloc(IP6_ARPA_MAX_LEN + 20);
 				c = 0;
 				for (i=0; i<(int)strlen(name); i++) {
@@ -795,6 +794,12 @@ main(int argc, char *argv[])
 			} else {
 				if (verbosity != -1) {
 					ldns_pkt_print(stdout, pkt);
+					if (ldns_pkt_tc(pkt)) {
+						fprintf(stdout,
+							"\n;; WARNING: The answer packet was truncated; you might want to\n");
+						fprintf(stdout,
+							";; query again with TCP (-t argument), or EDNS0 (-b for buffer size)\n");
+					}
 				}
 				if (qds) {
 					if (verbosity != -1) {
@@ -823,13 +828,34 @@ main(int argc, char *argv[])
 					}
 
 					/* verify */
-					result = ldns_pkt_verify(pkt, type, qname, key_list, NULL, NULL);
+#ifdef HAVE_SSL
+					key_verified = ldns_rr_list_new();
+					result = ldns_pkt_verify(pkt, type, qname, key_list, NULL, key_verified);
 
-					if (result == LDNS_STATUS_OK) {
+					if (result == LDNS_STATUS_ERR) {
+						/* is the existence denied then? */
+						result = ldns_verify_denial(pkt, qname, type, NULL, NULL);
+						if (result == LDNS_STATUS_OK) {
+							if (verbosity != -1) {
+								printf("Existence denied for ");
+								ldns_rdf_print(stdout, qname);
+								type_str = ldns_rr_type2str(type);
+								printf("\t%s\n", type_str);
+								LDNS_FREE(type_str);
+							}
+						} else {
+							if (verbosity != -1) {
+								printf("Bad data; RR for name and "
+								       "type not found or failed to "
+								       "verify, and denial of "
+								       "existence failed.\n");
+							}
+						}
+					} else if (result == LDNS_STATUS_OK) {
 						for(key_count = 0; key_count < ldns_rr_list_rr_count(key_verified);
 								key_count++) {
 							if (verbosity != -1) {
-								printf("VALIDATED by id = %d, owner = ",
+								printf("; VALIDATED by id = %d, owner = ",
 										(int)ldns_calc_keytag(
 												      ldns_rr_list_rr(key_verified, key_count)));
 								ldns_rdf_print(stdout, ldns_rr_owner(
@@ -841,8 +867,9 @@ main(int argc, char *argv[])
 						for(key_count = 0; key_count < ldns_rr_list_rr_count(key_list);
 								key_count++) {
 							if (verbosity != -1) {
-								printf("BOGUS by id = %d, owner = ",
-										(int)ldns_calc_keytag(
+								printf("; %s for id = %d, owner = ",
+								       ldns_get_errorstr_by_id(result),
+								       (int)ldns_calc_keytag(
 												      ldns_rr_list_rr(key_list, key_count)));
 								ldns_rdf_print(stdout, ldns_rr_owner(
 
@@ -851,9 +878,11 @@ main(int argc, char *argv[])
 								printf("\n");
 							}
 						}
-
 					}
-
+					ldns_rr_list_free(key_verified);
+#else
+					(void) key_count;
+#endif /* HAVE_SSL */
 				}
 				if (answer_file) {
 					dump_hex(pkt, answer_file);
@@ -884,6 +913,9 @@ main(int argc, char *argv[])
 	ERR_free_strings();
 	EVP_cleanup();
 #endif
+#ifdef USE_WINSOCK
+	WSACleanup();
+#endif
 
 	return result;
 }
