@@ -18,6 +18,10 @@
 #endif
 #include <errno.h>
 
+#ifndef IP_OFFMASK
+#define IP_OFFMASK 0x1fff
+#endif
+
 int verbosity = 1;
 
 #define ETHER_HEADER_LENGTH 14
@@ -88,6 +92,8 @@ enum enum_match_ids {
 	MATCH_DST_ADDRESS,
 	MATCH_TIMESTAMP,
 	MATCH_QUERY,
+	MATCH_QTYPE,
+	MATCH_QNAME,
 	MATCH_ANSWER,
 	MATCH_AUTHORITY,
 	MATCH_ADDITIONAL,
@@ -104,6 +110,7 @@ enum enum_counter_types {
 	TYPE_TIMESTAMP,
 	TYPE_ADDRESS,
 	TYPE_RR,
+	TYPE_RR_TYPE,
 	TYPE_LAST
 };
 typedef enum enum_counter_types counter_type;
@@ -143,7 +150,7 @@ const ldns_lookup_table lt_operators[] = {
 	{ 0, NULL }
 };
 
-const char *get_op_str(type_operator op) {
+static const char *get_op_str(type_operator op) {
 	const ldns_lookup_table *lt;
 	lt = ldns_lookup_by_id((ldns_lookup_table *) lt_operators, op);
 	if (lt) {
@@ -154,7 +161,7 @@ const char *get_op_str(type_operator op) {
 	}
 }
 
-type_operator
+static type_operator
 get_op_id(char *op_str)
 {
 	const ldns_lookup_table *lt;
@@ -183,6 +190,7 @@ const type_operators const_type_operators[] = {
 	{ TYPE_TIMESTAMP, 6, { OP_EQUAL, OP_NOTEQUAL, OP_GREATER, OP_LESSER, OP_GREATEREQUAL, OP_LESSEREQUAL, 0, 0, 0, 0 } },
 	{ TYPE_ADDRESS, 3, { OP_EQUAL, OP_NOTEQUAL, OP_CONTAINS, 0, 0, 0, 0, 0, 0, 0} },
 	{ TYPE_RR, 3, { OP_EQUAL, OP_NOTEQUAL, OP_CONTAINS, 0, 0, 0, 0, 0, 0, 0} },
+	{ TYPE_RR_TYPE, 6, { OP_EQUAL, OP_NOTEQUAL, OP_GREATER, OP_LESSER, OP_GREATEREQUAL, OP_LESSEREQUAL, 0, 0, 0, 0 } },
 	{ 0, 0, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } }
 };
 
@@ -211,6 +219,8 @@ typedef struct struct_match_table match_table;
  */
 const match_table matches[] = {
 	{ MATCH_QUERY, "query", "String representation of the query RR", TYPE_RR },
+	{ MATCH_QTYPE, "qtype", "RR Type of the question RR, if present", TYPE_RR_TYPE },
+	{ MATCH_QNAME, "qname", "Owner name of the question RR, if present", TYPE_STRING },
 	{ MATCH_SRC_ADDRESS, "srcaddress", "address the packet was sent from", TYPE_ADDRESS },
 	{ MATCH_TIMESTAMP, "timestamp", "time the packet was sent", TYPE_TIMESTAMP },
 	{ MATCH_DST_ADDRESS, "dstaddress", "address the packet was sent to", TYPE_ADDRESS },
@@ -285,7 +295,7 @@ get_match_by_name(char *name) {
 	return NULL;
 }
 
-match_table *
+static match_table *
 get_match_by_id(match_id id) {
 	match_table *mt = (match_table *) matches;
 
@@ -298,7 +308,7 @@ get_match_by_id(match_id id) {
 	return NULL;
 }
 
-const char *
+static const char *
 get_match_name_str(match_id id) {
 	match_table *mt = get_match_by_id(id);
 	if (mt) {
@@ -310,7 +320,8 @@ get_match_name_str(match_id id) {
 	}
 }
 
-bool is_match_name(char *name) {
+static bool
+is_match_name(char *name) {
 	match_table *mt = get_match_by_name(name);
 	if (mt) {
 		return true;
@@ -319,7 +330,7 @@ bool is_match_name(char *name) {
 	}
 }
 
-void
+static void
 print_match_operation(FILE *output, match_operation *mc)
 {
 	match_table *mt = NULL;
@@ -354,18 +365,18 @@ print_match_operation(FILE *output, match_operation *mc)
 					value = atoi(mc->value);
 					lt = ldns_lookup_by_id(ldns_opcodes, value);
 					if (lt) {
-						fprintf(output, lt->name);
+						fprintf(output, "%s", lt->name);
 					} else {
-						fprintf(output, mc->value);
+						fprintf(output, "%s", mc->value);
 					}
 					break;
 				case TYPE_RCODE:
 					value = atoi(mc->value);
 					lt = ldns_lookup_by_id(ldns_rcodes, value);
 					if (lt) {
-						fprintf(output, lt->name);
+						fprintf(output, "%s", lt->name);
 					} else {
-						fprintf(output, mc->value);
+						fprintf(output, "%s", mc->value);
 					}
 					break;
 				case TYPE_TIMESTAMP:
@@ -396,7 +407,7 @@ print_match_operation(FILE *output, match_operation *mc)
 	}
 }
 
-void
+static void
 print_match_expression(FILE *output, match_expression *expr)
 {
 	if (expr) {
@@ -442,7 +453,7 @@ fprintf(output, "_");
 	}			
 }
 
-size_t 
+static size_t 
 calculate_counters_total(match_counters *counters)
 {
 	size_t result = 0;
@@ -461,7 +472,7 @@ calculate_counters_total(match_counters *counters)
 	return result;
 }
 
-void
+static void
 print_counters(FILE *output, match_counters *counters, bool show_percentages, size_t total, int count_minimum)
 {
 	double percentage;
@@ -491,7 +502,7 @@ print_counters(FILE *output, match_counters *counters, bool show_percentages, si
 	return;	
 }
 
-void
+static void
 ldns_pkt2file_hex(FILE *fp, const ldns_pkt *pkt)
 {
 	uint8_t *wire;
@@ -529,7 +540,7 @@ ldns_pkt2file_hex(FILE *fp, const ldns_pkt *pkt)
  * (if they are 'under' this one in the tree, which should be the case in
  * the unique counter tree
  */
-size_t
+static size_t
 calculate_total_value(match_counters *counters, match_operation *cur)
 {
 	size_t result = 0;
@@ -552,7 +563,7 @@ calculate_total_value(match_counters *counters, match_operation *cur)
 	return result;
 }
 
-size_t
+static size_t
 calculate_total_count_matches(match_counters *counters, match_operation *cur)
 {
 	size_t result = 0;
@@ -589,7 +600,7 @@ calculate_total_count_matches(match_counters *counters, match_operation *cur)
  * Returns true if there is a previous match operation with the given type
  * in the counters structure
  */
-bool
+static bool
 has_previous_match(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -616,7 +627,7 @@ has_previous_match(match_counters *counters, match_operation *cur)
  * Returns true if there is a later match operation with the given type
  * in the counters structure
  */
-bool
+static bool
 has_next_match(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -643,7 +654,7 @@ has_next_match(match_counters *counters, match_operation *cur)
  * Returns the first match with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_first_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -665,7 +676,7 @@ get_first_match_expression(match_counters *counters, match_operation *cur)
  * Returns the second match expression with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_second_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -692,7 +703,7 @@ get_second_match_expression(match_counters *counters, match_operation *cur)
  * Returns the last match expression with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_last_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -714,7 +725,7 @@ get_last_match_expression(match_counters *counters, match_operation *cur)
  * Returns the last but one match expression with the same type at *cur in
  * the counter list, or NULL if it is not found
  */
-match_expression *
+static match_expression *
 get_last_but_one_match_expression(match_counters *counters, match_operation *cur)
 {
 	if (!counters) {
@@ -737,7 +748,7 @@ get_last_but_one_match_expression(match_counters *counters, match_operation *cur
 	}
 }
 
-size_t
+static size_t
 get_first_count(match_counters *counters, match_operation *cur)
 {
 	match_expression *o = get_first_match_expression(counters, cur);
@@ -748,7 +759,7 @@ get_first_count(match_counters *counters, match_operation *cur)
 	}
 }
 
-size_t
+static size_t
 get_last_count(match_counters *counters, match_operation *cur)
 {
 	match_expression *o = get_last_match_expression(counters, cur);
@@ -760,7 +771,7 @@ get_last_count(match_counters *counters, match_operation *cur)
 }
 
 
-size_t
+static size_t
 calculate_total_count(match_counters *counters, match_operation *cur)
 {
 	size_t result = 0;
@@ -783,7 +794,7 @@ calculate_total_count(match_counters *counters, match_operation *cur)
 	return result;
 }
 
-void
+static void
 print_counter_averages(FILE *output, match_counters *counters, match_operation *cur)
 {
 	size_t total_value;
@@ -822,7 +833,7 @@ print_counter_averages(FILE *output, match_counters *counters, match_operation *
 	return;	
 }
 
-void
+static void
 print_counter_average_count(FILE *output, match_counters *counters, match_operation *cur, bool remove_first_last)
 {
 	size_t total_matches;
@@ -881,7 +892,7 @@ print_counter_average_count(FILE *output, match_counters *counters, match_operat
 	return;	
 }
 
-bool
+static bool
 match_int(type_operator operator,
           char *value,
 	  char *mvalue)
@@ -920,10 +931,11 @@ match_int(type_operator operator,
 	}
 }
 
-bool
+static bool
 match_opcode(type_operator operator,
              char *value,
-             char *mvalue) {
+             char *mvalue)
+{
 	ldns_pkt_opcode a, b;
 	int i;
 	ldns_lookup_table *lt;
@@ -985,7 +997,7 @@ match_opcode(type_operator operator,
 	}
 }
 
-bool
+static bool
 match_str(type_operator operator,
           char *value,
           char *mvalue)
@@ -1017,10 +1029,46 @@ match_str(type_operator operator,
 	}	
 }
 
-bool
+static bool
+match_rr_type(type_operator operator,
+              char *value,
+	      char *mvalue)
+{
+	ldns_rr_type a,b;
+	
+	a = ldns_get_rr_type_by_name(value);
+	b = ldns_get_rr_type_by_name(mvalue);
+	
+	switch (operator) {
+		case OP_EQUAL:
+			return a == b;
+			break;
+		case OP_NOTEQUAL:
+			return a != b;
+			break;
+		case OP_GREATER:
+			return a > b;
+			break;
+		case OP_LESSER:
+			return a < b;
+			break;
+		case OP_GREATEREQUAL:
+			return a >= b;
+			break;
+		case OP_LESSEREQUAL:
+			return a <= b;
+			break;
+		default:
+			fprintf(stderr, "Unknown operator: %u\n", operator);
+			exit(2);
+	}
+}
+
+static bool
 match_rcode(type_operator operator,
              char *value,
-             char *mvalue) {
+             char *mvalue)
+{
 	int a, b;
 	int i;
 	ldns_lookup_table *lt;
@@ -1083,7 +1131,7 @@ match_rcode(type_operator operator,
 	}
 }
 
-bool
+static bool
 value_matches(match_id id,
         type_operator operator,
         char *value,
@@ -1119,6 +1167,7 @@ value_matches(match_id id,
 			result = match_int(operator, value, mvalue);
 			break;
 		case MATCH_QUERY:
+		case MATCH_QNAME:
 		case MATCH_ANSWER:
 		case MATCH_AUTHORITY:
 		case MATCH_ADDITIONAL:
@@ -1128,6 +1177,9 @@ value_matches(match_id id,
 		case MATCH_DST_ADDRESS:
 			result = match_str(operator, value, mvalue);
 			break;
+		case MATCH_QTYPE:
+			result = match_rr_type(operator, value, mvalue);
+			break;
 		default:
 			fprintf(stderr, "Error: value_matches() for operator %s not implemented yet.\n", get_op_str((type_operator) id));
 			exit(3);
@@ -1141,64 +1193,8 @@ value_matches(match_id id,
 	}
 	return result;
 }
-	
 
-#if 0
-bool
-count_match(match_counters *counters,
-            match_id id,
-	    char *value
-	   )
-{
-	size_t i;
-	match_table *mt;
-
-	if (counters) {
-		for(i = 0; i < counters->size; i++) {
-			if (id == counters->counter[i].id) {
-				if (value) {
-					if (value_matches(id, 
-					                  counters->counter[i].operator,
-					                  value,
-					                  counters->counter[i].value)) {
-
-						if (verbosity >= 5) {
-							mt = get_match_by_id(counters->counter[i].id);
-							if (mt) {
-								printf("MATCH: %s: %s %s %s\n", mt->name, value, get_op_str(counters->counter[i].operator), counters->counter[i].value);
-							} else {
-								printf("MATCH: <unknown?>: %s %s %s\n", value, get_op_str(counters->counter[i].operator), counters->counter[i].value);
-							}
-						}
-						counters->counter[i].count++;
-						return true;
-					}
-				} else {
-					/*counters->counter[i].count++;*/
-				}
-			}
-		}
-	}
-	return false;
-}
-
-/* if value == NULL, always count */
-bool
-count_match_i(match_counters *counters,
-              match_id id,
-              int value)
-{
-	char intbuf[20];
-	bool result;
-
-	memset(intbuf, 0, 10);
-	snprintf(intbuf, 20, "%d", value);
-	result = count_match(counters, id, intbuf);
-	return result;
-}
-#endif
-
-char *
+static char *
 get_string_value(match_id id, ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr)
 {
 	char *val;
@@ -1280,6 +1276,26 @@ get_string_value(match_id id, ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_a
 				val[0] = '\0';
 			}
 			break;
+		case MATCH_QNAME:
+			if (ldns_pkt_qdcount(pkt) > 0) {
+				free(val);
+				val = ldns_rdf2str(ldns_rr_owner(ldns_rr_list_rr(ldns_pkt_question(pkt), 0)));
+				/* replace \n for nicer printing later */
+				if (strchr(val, '\n')) {
+					*(strchr(val, '\n')) = '\0';
+				}
+			} else {
+				val[0] = '\0';
+			}
+			break;
+		case MATCH_QTYPE:
+			if (ldns_pkt_qdcount(pkt) > 0) {
+				free(val);
+				val = ldns_rr_type2str(ldns_rr_get_type(ldns_rr_list_rr(ldns_pkt_question(pkt), 0)));
+			} else {
+				val[0] = '\0';
+			}
+			break;
 		case MATCH_ANSWER:
 			if (ldns_pkt_ancount(pkt) > 0) {
 				free(val);
@@ -1318,30 +1334,7 @@ get_string_value(match_id id, ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_a
 	return val;
 }
 
-/*
-bool
-match_pkt(ldns_pkt *pkt, match_counter *counter)
-{
-	bool result;
-	char *val;
-
-	if (!pkt || !counter) {
-		return false;
-	} else {
-		val = get_string_value(counter->id, pkt);
-		if (!val) {
-			return false;
-		}
-		result = value_matches(counter->id, counter->operator, val, counter->value);
-		if (result) {
-			counter->count++;
-		}
-		free(val);
-		return result;
-	}
-}
-*/
-bool
+static bool
 match_packet_to_operation(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_operation *operation)
 {
 	bool result;
@@ -1360,22 +1353,7 @@ match_packet_to_operation(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr,
 	}
 }
 
-
-
-/*
-int
-add_counter(match_counters *counters,
-            match_expression *expr)
-{
-		counters->counter = realloc(counters->counter, (counters->size + 1) * sizeof(match_expression *));
-		counters->counter[counters->size] = expr;
-		counters->size = counters->size + 1;
-
-		return 0;
-}
-*/
-
-int
+static int
 match_expression_compare_count(const void *a, const void *b)
 {
 	match_expression *mea, *meb;
@@ -1398,7 +1376,7 @@ match_expression_compare_count(const void *a, const void *b)
 	}
 }
 
-int
+static int
 match_expression_compare_count_p(const void *a, const void *b)
 {
 	match_expression **pmea, **pmeb;
@@ -1414,7 +1392,7 @@ match_expression_compare_count_p(const void *a, const void *b)
 	}
 }
 
-int
+static int
 match_operation_compare(const void *a, const void *b)
 {
 	match_operation *moa, *mob;
@@ -1466,7 +1444,7 @@ match_operation_compare(const void *a, const void *b)
 	}
 }
 
-int
+static int
 match_expression_compare(const void *a, const void *b)
 {
 	match_expression *mea, *meb;
@@ -1505,7 +1483,8 @@ match_expression_compare(const void *a, const void *b)
 		}
 	}
 }
-int
+
+static int
 match_expression_compare_p(const void *a, const void *b)
 {
 	
@@ -1525,7 +1504,7 @@ match_expression_compare_p(const void *a, const void *b)
 /**
  * If count is true, and the counter is found, its count is increased by 1
  */
-int
+static int
 add_match_counter(match_counters *counters,
 		  match_expression *expr,
                   bool count)
@@ -1580,7 +1559,7 @@ add_match_counter(match_counters *counters,
 	return 0;
 }
 
-bool
+static bool
 match_dns_packet_to_expr(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_expression *expr)
 {
 	bool result;
@@ -1621,7 +1600,7 @@ match_dns_packet_to_expr(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr,
 	return result;
 }
 
-bool
+static bool
 match_expression_equals(match_expression *expr1, match_expression *expr2)
 {
 	if (!expr1 || !expr2) {
@@ -1652,8 +1631,7 @@ match_expression_equals(match_expression *expr1, match_expression *expr2)
 
 }
 
-
-void
+static void
 free_match_operation(match_operation *operation)
 {
 	if (operation) {
@@ -1664,7 +1642,7 @@ free_match_operation(match_operation *operation)
 	}
 }
 
-void
+static void
 free_match_expression(match_expression *expr)
 {
 	if (expr) {
@@ -1682,7 +1660,7 @@ free_match_expression(match_expression *expr)
 	}
 }
 
-void
+static void
 free_counters(match_counters *counters)
 {
 	if (counters) {
@@ -1699,7 +1677,7 @@ free_counters(match_counters *counters)
 	}
 }
 
-void
+static void
 match_pkt_counters(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_counters *counts)
 {
 	if (counts->left) {
@@ -1717,7 +1695,7 @@ match_pkt_counters(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_
 	}	
 }
 
-void
+static void
 match_pkt_uniques(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_counters *uniques, match_id unique_ids[], size_t unique_id_count)
 {
 	match_expression *me;
@@ -1780,26 +1758,7 @@ match_pkt_uniques(ldns_pkt *pkt, ldns_rdf *src_addr, ldns_rdf *dst_addr, match_c
 #endif
 }
 
-
-/*
-void
-print_match_counter(FILE *output, match_counter *mc)
-{
-	match_table *mt = NULL;
-	if (mc) {
-		mt = get_match_by_id(mc->id);
-		if (mt) {
-			fprintf(output, "%s %s '%s'", mt->name, get_op_str(mc->operator), mc->value);
-		} else {
-			fprintf(output, "%u %s '%s'", mc->id, get_op_str(mc->operator), mc->value);
-		}
-	} else {
-		fprintf(output, "(nil)");
-	}
-}
-
-*/
-match_expression *
+static match_expression *
 parse_match_expression(char *string)
 {
 	match_expression *expr;
@@ -1817,21 +1776,13 @@ parse_match_expression(char *string)
 
 	j = 0;
 	for (i = 0; i < strlen(string); i++) {
-/*
 		if(!isspace(string[i])) {
-*/
 			str[j] = string[i];
 			j++;
-/*
 		}
-*/
 	}
 	str[j] = '\0';
 	
-	/*
-	printf("Parsing: %s\n", string);
-	printf("Parsing short : %s\n", str);
-	*/
 	expr = malloc(sizeof(match_expression));
 	expr->left = NULL;
 	expr->right = NULL;
@@ -2204,7 +2155,7 @@ handle_ether_packet(const u_char *data, struct pcap_pkthdr cur_hdr, match_counte
 	struct ip *iptr;
 	struct ip6_hdr *ip6_hdr;
 	int ip_hdr_size;
-	u_int8_t protocol;
+	uint8_t protocol;
 	size_t data_offset = 0;
 	ldns_rdf *src_addr, *dst_addr;
 	uint8_t *ap;
@@ -2316,7 +2267,7 @@ printf("timeval: %u ; %u\n", cur_hdr.ts.tv_sec, cur_hdr.ts.tv_usec);
 				printf("\t; ??? - %u\n", ip_len);
 */
 			} else {
-				printf("Lost fragment %u\n", iptr->ip_id);
+				/*printf("Lost fragment %u\n", iptr->ip_id);*/
 				lost_packet_fragments++;
 				return 1;
 			}
@@ -2349,7 +2300,7 @@ printf("timeval: %u ; %u\n", cur_hdr.ts.tv_sec, cur_hdr.ts.tv_usec);
 			}
 
 			ip_hdr_size = (int) iptr->ip_hl * 4;
-			protocol = (u_int8_t) iptr->ip_p;
+			protocol = (uint8_t) iptr->ip_p;
 			
 			data_offset += ip_hdr_size;
 
@@ -2490,7 +2441,7 @@ printf("timeval: %u ; %u\n", cur_hdr.ts.tv_sec, cur_hdr.ts.tv_usec);
 		}
 
 		ip_hdr_size = IP6_HEADER_LENGTH;
-		protocol = (u_int8_t) ip6_hdr->ip6_ctlun.ip6_un1.ip6_un1_nxt;
+		protocol = (uint8_t) ip6_hdr->ip6_ctlun.ip6_un1.ip6_un1_nxt;
 		
 		data_offset += ip_hdr_size;
 
@@ -2577,7 +2528,7 @@ printf("timeval: %u ; %u\n", cur_hdr.ts.tv_sec, cur_hdr.ts.tv_usec);
 			/* tcp packets are skipped */
 			tcp_packets++;
 		} else {
-			printf("ipv6 unknown next header type: %u\n", protocol);
+			printf("ipv6 unknown next header type: %u\n", (unsigned int) protocol);
 		}
 
 
