@@ -18,11 +18,49 @@
 
 #ifdef HAVE_SSL
 #include <openssl/err.h>
-#endif
 
 int verbosity = 3;
 
-bool
+/* returns 1 if the list is empty, or if there are only ns rrs in the
+ * list, 0 otherwise */
+static int
+only_ns_in_rrsets(ldns_dnssec_rrsets *rrsets) {
+	ldns_dnssec_rrsets *cur_rrset = rrsets;
+
+	while (cur_rrset) {
+		if (cur_rrset->type != LDNS_RR_TYPE_NS) {
+			return 0;
+		}
+		cur_rrset = cur_rrset->next;
+	}
+	return 1;
+}
+
+static int
+zone_is_nsec3_optout(ldns_rbtree_t *zone_nodes)
+{
+	/* simply find the first NSEC3 RR and check its flags */
+	/* TODO: maybe create a general function that uses the active
+	 * NSEC3PARAM RR? */
+	ldns_rbnode_t *cur_node;
+	ldns_dnssec_name *cur_name;
+	cur_node = ldns_rbtree_first(zone_nodes);
+	while (cur_node != LDNS_RBTREE_NULL) {
+		cur_name = (ldns_dnssec_name *) cur_node->data;
+		if (cur_name && cur_name->nsec &&
+		    ldns_rr_get_type(cur_name->nsec) == LDNS_RR_TYPE_NSEC3) {
+			if (ldns_nsec3_optout(cur_name->nsec)) {
+				return 1;
+			} else {
+				return 0;
+			}
+		}
+		cur_node = ldns_rbtree_next(cur_node);
+	}
+	return 0;
+}
+
+static bool
 ldns_rr_list_contains_name(const ldns_rr_list *rr_list,
 					  const ldns_rdf *name)
 {
@@ -39,7 +77,7 @@ ldns_rr_list_contains_name(const ldns_rr_list *rr_list,
 	return false;
 }
 
-void
+static void
 print_type(ldns_rr_type type)
 {
 	const ldns_rr_descriptor *descriptor;
@@ -54,7 +92,7 @@ print_type(ldns_rr_type type)
 
 }
 
-ldns_dnssec_zone *
+static ldns_dnssec_zone *
 create_dnssec_zone(ldns_zone *orig_zone)
 {
 	size_t i;
@@ -105,11 +143,12 @@ create_dnssec_zone(ldns_zone *orig_zone)
 	return dnssec_zone;
 }
 
-ldns_status
+static ldns_status
 verify_dnssec_rrset(ldns_rdf *zone_name,
 					ldns_rdf *name,
                     ldns_dnssec_rrsets *rrset,
-                    ldns_rr_list *keys)
+                    ldns_rr_list *keys,
+                    ldns_rr_list *glue_rrs)
 {
 	ldns_rr_list *rrset_rrs;
 	ldns_dnssec_rrs *cur_rr, *cur_sig;
@@ -117,9 +156,11 @@ verify_dnssec_rrset(ldns_rdf *zone_name,
 	ldns_rr_list *good_keys;
 	ldns_status result = LDNS_STATUS_OK;
 	
+	if (!rrset->rrs) return LDNS_STATUS_OK;
+	
 	rrset_rrs = ldns_rr_list_new();
 	cur_rr = rrset->rrs;
-	while(cur_rr) {
+	while(cur_rr && cur_rr->rr) {
 		ldns_rr_list_push_rr(rrset_rrs, cur_rr->rr);
 		cur_rr = cur_rr->next;
 	}
@@ -144,12 +185,10 @@ verify_dnssec_rrset(ldns_rdf *zone_name,
 					if (result == LDNS_STATUS_OK) {
 						result = status;
 					}
-#ifdef HAVE_SSL
 					if (status == LDNS_STATUS_SSL_ERR) {
 						ERR_load_crypto_strings();
 						ERR_print_errors_fp(stdout);
 					}
-#endif
 					if (verbosity >= 4) {
 						printf("RRSet:\n");
 						ldns_dnssec_rrs_print(stdout, rrset->rrs);
@@ -180,7 +219,7 @@ verify_dnssec_rrset(ldns_rdf *zone_name,
 	return result;
 }
 
-ldns_status
+static ldns_status
 verify_single_rr(ldns_rr *rr,
 			  ldns_dnssec_rrs *signature_rrs,
 			  ldns_rr_list *keys)
@@ -212,12 +251,10 @@ verify_single_rr(ldns_rr *rr,
 				printf("\t");
 				print_type(ldns_rr_get_type(rr));
 				printf("\n");
-#ifdef HAVE_SSL
 				if (status == LDNS_STATUS_SSL_ERR) {
 					ERR_load_crypto_strings();
 					ERR_print_errors_fp(stdout);
 				}
-#endif
 				if (verbosity >= 4) {
 					printf("RRSet:\n");
 					ldns_rr_list_print(stdout, rrset_rrs);
@@ -237,23 +274,236 @@ verify_single_rr(ldns_rr *rr,
 	return result;
 }
 
-ldns_status
+static ldns_status
+verify_next_hashed_name(ldns_rbtree_t *zone_nodes,
+                        ldns_dnssec_name *name)
+{
+	ldns_rbnode_t *next_node;
+	ldns_dnssec_name *next_name;
+	ldns_dnssec_name *cur_next_name = NULL;
+	ldns_dnssec_name *cur_first_name = NULL;
+	int cmp;
+	char *next_owner_str;
+	ldns_rdf *next_owner_dname;
+	
+	if (!name->hashed_name) {
+		name->hashed_name = ldns_nsec3_hash_name_frm_nsec3(name->nsec,
+		                                                   name->name);
+	}
+	next_node = ldns_rbtree_first(zone_nodes);
+	while (next_node != LDNS_RBTREE_NULL) {
+		next_name = (ldns_dnssec_name *)next_node->data;
+		/* skip over names that have no NSEC3 records (whether it
+		 * actually should or should not should have been checked
+		 * already */
+		if (!next_name->nsec) {
+			next_node = ldns_rbtree_next(next_node);
+			continue;
+		}
+		if (!next_name->hashed_name) {
+			next_name->hashed_name = ldns_nsec3_hash_name_frm_nsec3(
+			                              name->nsec, next_name->name);
+		}
+		/* we keep track of what 'so far' is the next hashed name;
+		 * it must of course be 'larger' than the current name
+		 * if we find one that is larger, but smaller than what we
+		 * previously thought was the next one, that one is the next
+		 */
+		cmp = ldns_dname_compare(name->hashed_name,
+		                         next_name->hashed_name);
+		if (cmp < 0) {
+			if (!cur_next_name) {
+				cur_next_name = next_name;
+			} else {
+				cmp = ldns_dname_compare(next_name->hashed_name,
+				                         cur_next_name->hashed_name);
+				if (cmp < 0) {
+					cur_next_name = next_name;
+				}
+			}
+		}
+		/* in case the hashed name of the nsec we are checking is the
+		 * last one, we need the first hashed name of the zone */
+		if (!cur_first_name) {
+			cur_first_name = next_name;
+		} else {
+			cmp = ldns_dname_compare(next_name->hashed_name,
+									 cur_first_name->hashed_name);
+			if (cmp < 0) {
+				cur_first_name = next_name;
+			}
+		}
+		next_node = ldns_rbtree_next(next_node);
+	}
+	if (!cur_next_name) {
+		cur_next_name = cur_first_name;
+	}
+	
+	next_owner_str = ldns_rdf2str(ldns_nsec3_next_owner(name->nsec));
+	next_owner_dname = ldns_dname_new_frm_str(next_owner_str);
+	cmp = ldns_dname_compare(next_owner_dname,
+	                         cur_next_name->hashed_name);
+	ldns_rdf_deep_free(next_owner_dname);
+	LDNS_FREE(next_owner_str);
+	if (cmp != 0) {
+		printf("Error: The NSEC3 record for ");
+		ldns_rdf_print(stdout, name->name);
+		printf(" points to the wrong next hashed owner name\n");
+		printf("(should point to ");
+		ldns_rdf_print(stdout, cur_next_name->name);
+		printf("(whose hashed name is ");
+		ldns_rdf_print(stdout, cur_next_name->hashed_name);
+		printf(")\n");
+		return LDNS_STATUS_ERR;
+	} else {
+		return LDNS_STATUS_OK;
+	}
+}
+
+static ldns_rbnode_t *
+next_nonglue_node(ldns_rbnode_t *node, ldns_rr_list *glue_rrs)
+{
+	ldns_rbnode_t *cur_node = ldns_rbtree_next(node);
+	ldns_dnssec_name *cur_name;
+	while (cur_node != LDNS_RBTREE_NULL) {
+		cur_name = (ldns_dnssec_name *) cur_node->data;
+		if (cur_name && cur_name->name) {
+			if (!ldns_rr_list_contains_name(glue_rrs, cur_name->name)) {
+				return cur_node;
+			}
+		}
+		cur_node = ldns_rbtree_next(cur_node);
+	}
+	return LDNS_RBTREE_NULL;
+}
+
+static ldns_status
+verify_nsec(ldns_rbtree_t *zone_nodes,
+            ldns_rbnode_t *cur_node,
+            ldns_rr_list *keys,
+            ldns_rr_list *glue_rrs
+)
+{
+	ldns_rbnode_t *next_node;
+	ldns_dnssec_name *name, *next_name;
+	ldns_status status, result;
+	result = LDNS_STATUS_OK;
+	
+	name = (ldns_dnssec_name *) cur_node->data;
+	if (name->nsec) {
+		if (name->nsec_signatures) {
+			status = verify_single_rr(name->nsec,
+								 name->nsec_signatures,
+								 keys);
+			if (result == LDNS_STATUS_OK) {
+				result = status;
+			}
+		} else {
+			if (verbosity >= 1) {
+				printf("Error: the NSEC(3) record of ");
+				ldns_rdf_print(stdout, name->name);
+				printf(" has no signatures\n");
+			}
+			if (result == LDNS_STATUS_OK) {
+				result = LDNS_STATUS_ERR;
+			}
+		}
+		/* check whether the NSEC record points to the right name */
+		switch (ldns_rr_get_type(name->nsec)) {
+			case LDNS_RR_TYPE_NSEC:
+				/* simply try next name */
+				next_node = next_nonglue_node(cur_node, glue_rrs);
+				if (next_node == LDNS_RBTREE_NULL) {
+					next_node = ldns_rbtree_first(zone_nodes);
+				}
+				next_name = (ldns_dnssec_name *)next_node->data;
+				if (ldns_dname_compare(next_name->name,
+									   ldns_rr_rdf(name->nsec, 0))
+					!= 0) {
+					printf("Error: the NSEC record for ");
+					ldns_rdf_print(stdout, name->name);
+					printf(" points to the wrong next owner name\n");
+					if (result == LDNS_STATUS_OK) {
+						result = LDNS_STATUS_ERR;
+					}
+				}
+				break;
+			case LDNS_RR_TYPE_NSEC3:
+				/* find the hashed next name in the tree */
+				/* this is expensive, do we need to add support
+				 * for this in the structs? (ie. pointer to next
+				 * hashed name?)
+				 */
+				status = verify_next_hashed_name(zone_nodes, name);
+				if (result == LDNS_STATUS_OK) {
+					result = status;
+				}
+				break;
+			default:
+				break;
+		}
+		
+	} else {
+		/* todo; do this once and cache result? */
+		if (zone_is_nsec3_optout(zone_nodes) &&
+		    only_ns_in_rrsets(name->rrsets)) {
+			/* ok, no problem, but we need to remember to check
+			 * whether the chain does not actually point to this
+			 * name later */
+		} else {
+			if (verbosity >= 1) {
+				printf("Error: there is no NSEC(3) for ");
+				ldns_rdf_print(stdout, name->name);
+				printf("\n");
+			}
+			if (result == LDNS_STATUS_OK) {
+				result = LDNS_STATUS_ERR;
+			}
+		}
+	}
+	return result;
+}
+
+static int
+ldns_dnssec_name_has_only_a(ldns_dnssec_name *cur_name)
+{
+	ldns_dnssec_rrsets *cur_rrset;
+	cur_rrset = cur_name->rrsets;
+	while (cur_rrset) {
+		if (cur_rrset->type != LDNS_RR_TYPE_A &&
+			cur_rrset->type != LDNS_RR_TYPE_AAAA) {
+			return 0;
+		} else {
+			cur_rrset = cur_rrset->next;
+		}
+	}
+	return 1;
+}
+
+static ldns_status
 verify_dnssec_name(ldns_rdf *zone_name,
-                ldns_dnssec_name *name,
+                ldns_dnssec_zone *zone,
+                ldns_rbtree_t *zone_nodes,
+                ldns_rbnode_t *cur_node,
 			    ldns_rr_list *keys,
 			    ldns_rr_list *glue_rrs)
 {
 	ldns_status result = LDNS_STATUS_OK;
 	ldns_status status;
 	ldns_dnssec_rrsets *cur_rrset;
+	ldns_dnssec_name *name;
+	/* for NSEC chain checks */
 
+	name = (ldns_dnssec_name *) cur_node->data;
 	if (verbosity >= 3) {
 		printf("Checking: ");
 		ldns_rdf_print(stdout, name->name);
 		printf("\n");
 	}
 
-	if (ldns_rr_list_contains_name(glue_rrs, name->name)) {
+	if (ldns_rr_list_contains_name(glue_rrs, name->name) &&
+		ldns_dnssec_name_has_only_a(name)
+	) {
 		/* glue */
 		cur_rrset = name->rrsets;
 		while (cur_rrset) {
@@ -281,46 +531,25 @@ verify_dnssec_name(ldns_rdf *zone_name,
 		/* not glue, do real verify */
 		cur_rrset = name->rrsets;
 		while(cur_rrset) {
-			status = verify_dnssec_rrset(zone_name, name->name, cur_rrset, keys);
-			if (status != LDNS_STATUS_OK && result == LDNS_STATUS_OK) {
-				result = status;
+			if (cur_rrset->type != LDNS_RR_TYPE_A ||
+			    !ldns_dnssec_zone_find_rrset(zone, name->name, LDNS_RR_TYPE_NS)) {
+				status = verify_dnssec_rrset(zone_name, name->name, cur_rrset, keys, glue_rrs);
+				if (status != LDNS_STATUS_OK && result == LDNS_STATUS_OK) {
+					result = status;
+				}
 			}
 			cur_rrset = cur_rrset->next;
 		}
 
-		if (name->nsec) {
-			if (name->nsec_signatures) {
-				status = verify_single_rr(name->nsec,
-									 name->nsec_signatures,
-									 keys);
-				if (result == LDNS_STATUS_OK) {
-					result = status;
-				}
-			} else {
-				if (verbosity >= 1) {
-					printf("Error: the NSEC(3) record of ");
-					ldns_rdf_print(stdout, name->name);
-					printf(" has no signatures\n");
-				}
-				if (result == LDNS_STATUS_OK) {
-					result = LDNS_STATUS_ERR;
-				}
-			}
-		} else {
-			if (verbosity >= 1) {
-				printf("Error: there is no NSEC(3) for ");
-				ldns_rdf_print(stdout, name->name);
-				printf("\n");
-			}
-			if (result == LDNS_STATUS_OK) {
-				result = LDNS_STATUS_ERR;
-			}
+		status = verify_nsec(zone_nodes, cur_node, keys, glue_rrs);
+		if (result == LDNS_STATUS_OK) {
+			result = status;
 		}
 	}
 	return result;
 }
 
-ldns_status
+static ldns_status
 verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
 			    ldns_rdf *zone_name,
 			    ldns_rr_list *glue_rrs)
@@ -358,8 +587,10 @@ verify_dnssec_zone(ldns_dnssec_zone *dnssec_zone,
 		}
 		while (cur_node != LDNS_RBTREE_NULL) {
 			cur_name = (ldns_dnssec_name *) cur_node->data;
-			status = verify_dnssec_name(zone_name, 
-			                            cur_name,
+			status = verify_dnssec_name(zone_name,
+			                            dnssec_zone,
+			                            dnssec_zone->names,
+			                            cur_node,
 			                            keys,
 			                            glue_rrs);
 			if (status != LDNS_STATUS_OK && result == LDNS_STATUS_OK) {
@@ -393,7 +624,7 @@ main(int argc, char **argv)
 			printf("\tReads the zonefile and checks for DNSSEC errors.\n");
 			printf("\nIt checks whether NSEC(3)s are present,");
 			printf(" and verifies all signatures\n");
-			printf("It does NOT check the NSEC(3) chain itself\n");
+			printf("It also checks the NSEC(3) chain, but it will error on opted-out delegations\n");
 			printf("\nOPTIONS:\n");
 			printf("\t-h show this text\n");
 			printf("\t-v shows the version and exits\n");
@@ -432,6 +663,11 @@ main(int argc, char **argv)
 	
 	s = ldns_zone_new_frm_fp_l(&z, fp, NULL, 0, LDNS_RR_CLASS_IN, &line_nr);
 
+	if (!ldns_zone_soa(z)) {
+		fprintf(stderr, "; Error: no SOA in the zone\n");
+		exit(1);
+	}
+
 	glue_rrs = ldns_zone_glue_rr_list(z);
 	if (s == LDNS_STATUS_OK) {
 		dnssec_zone = create_dnssec_zone(z);
@@ -455,8 +691,8 @@ main(int argc, char **argv)
 			}
 		}
 
-		ldns_zone_deep_free(z);
-		ldns_dnssec_zone_free(dnssec_zone);
+		ldns_zone_free(z);
+		ldns_dnssec_zone_deep_free(dnssec_zone);
 	} else {
 		fprintf(stderr, "%s at %d\n", 
 				ldns_get_errorstr_by_id(s),
@@ -467,4 +703,11 @@ main(int argc, char **argv)
 
 	exit(result);
 }
-
+#else
+int
+main(int argc, char **argv)
+{
+	fprintf(stderr, "ldns-verifyzone needs OpenSSL support, which has not been compiled in\n");
+	return 1;
+}
+#endif /* HAVE_SSL */
