@@ -149,7 +149,7 @@ ldns_cert_algorithm2buffer_str(ldns_buffer *output,
 }
 
 char *
-ldns_pkt_opcode2str(ldns_pkt_rcode opcode)
+ldns_pkt_opcode2str(ldns_pkt_opcode opcode)
 {
 	char *str;
 	ldns_buffer *buf;
@@ -201,7 +201,7 @@ ldns_pkt_algorithm2str(ldns_algorithm algorithm)
 }
 
 char *
-ldns_pkt_cert_algorithm2str(ldns_algorithm cert_algorithm)
+ldns_pkt_cert_algorithm2str(ldns_cert_algorithm cert_algorithm)
 {
 	char *str;
 	ldns_buffer *buf;
@@ -251,8 +251,7 @@ ldns_rdf2buffer_str_dname(ldns_buffer *output, const ldns_rdf *dname)
 				    data[src_pos] == '(' || data[src_pos] == ')') {
 					ldns_buffer_printf(output, "\\%c", 
 							data[src_pos]);
-					/* isprint!? */
-				} else if (!isprint((int) data[src_pos])) {
+				} else if (!isgraph((int) data[src_pos])) {
 					ldns_buffer_printf(output, "\\%03u", 
 							data[src_pos]);
 				} else {
@@ -345,7 +344,7 @@ ldns_rdf2buffer_str_str(ldns_buffer *output, const ldns_rdf *rdf)
 	ldns_buffer_printf(output, "\"");
 	for (i = 1; i <= length; ++i) {
 		char ch = (char) data[i];
-		if (isprint((int)ch)) {
+		if (isprint((int)ch) || ch=='\t') {
 			if (ch == '"' || ch == '\\') {
 				ldns_buffer_printf(output, "\\");
 			}
@@ -571,7 +570,7 @@ ldns_rdf2buffer_str_loc(ldns_buffer *output, const ldns_rdf *rdf)
 	uint8_t vertical_precision;
 	uint32_t longitude;
 	uint32_t latitude;
-	long altitude;
+	uint32_t altitude;
 	char northerness;
 	char easterness;
 	uint32_t h;
@@ -1078,10 +1077,16 @@ ldns_rr2buffer_str(ldns_buffer *output, const ldns_rr *rr)
 		}
 
 		ldns_buffer_printf(output, "\t");
-		ldns_rr_class2buffer_str(output, ldns_rr_get_class(rr));
+		status = ldns_rr_class2buffer_str(output, ldns_rr_get_class(rr));
+		if (status != LDNS_STATUS_OK) {
+			return status;
+		}
 		ldns_buffer_printf(output, "\t");
 
-		ldns_rr_type2buffer_str(output, ldns_rr_get_type(rr));
+		status = ldns_rr_type2buffer_str(output, ldns_rr_get_type(rr));
+		if (status != LDNS_STATUS_OK) {
+			return status;
+		}
 		
 		if (ldns_rr_rd_count(rr) > 0) {
 			ldns_buffer_printf(output, "\t");
@@ -1098,26 +1103,26 @@ ldns_rr2buffer_str(ldns_buffer *output, const ldns_rr *rr)
 		if (ldns_rr_rd_count(rr) > 0) {
 			switch (ldns_rr_get_type(rr)) {
 				case LDNS_RR_TYPE_DNSKEY:
-#ifdef HAVE_SSL
-					flags = ldns_rdf2native_int16(ldns_rr_rdf(rr, 0));
-					if (flags == 256 || flags == 384) {
-						ldns_buffer_printf(output, 
-								" ;{id = %d (zsk), size = %db}", 
+					if (ldns_rr_rdf(rr, 0)) {
+						flags = ldns_rdf2native_int16(ldns_rr_rdf(rr, 0));
+						if (flags == 256 || flags == 384) {
+							ldns_buffer_printf(output, 
+									" ;{id = %d (zsk), size = %db}", 
+									ldns_calc_keytag(rr),
+									ldns_rr_dnskey_key_size(rr)); 
+							break;
+						} 
+						if (flags == 257 || flags == 385) {
+							ldns_buffer_printf(output, 
+									" ;{id = %d (ksk), size = %db}", 
+									ldns_calc_keytag(rr),
+									ldns_rr_dnskey_key_size(rr)); 
+							break;
+						} 
+						ldns_buffer_printf(output, " ;{id = %d, size = %db}", 
 								ldns_calc_keytag(rr),
 								ldns_rr_dnskey_key_size(rr)); 
-						break;
-					} 
-					if (flags == 257 || flags == 385) {
-						ldns_buffer_printf(output, 
-								" ;{id = %d (ksk), size = %db}", 
-								ldns_calc_keytag(rr),
-								ldns_rr_dnskey_key_size(rr)); 
-						break;
-					} 
-					ldns_buffer_printf(output, " ;{id = %d, size = %db}", 
-							ldns_calc_keytag(rr),
-							ldns_rr_dnskey_key_size(rr)); 
-#endif /* HAVE_SSL */
+					}
 					break;
 				case LDNS_RR_TYPE_RRSIG:
 					ldns_buffer_printf(output, " ;{id = %d}", 
@@ -1326,6 +1331,7 @@ ldns_pkt2buffer_str(ldns_buffer *output, const ldns_pkt *pkt)
 	return status;
 }
 
+#ifdef HAVE_SSL
 static ldns_status
 ldns_hmac_key2buffer_str(ldns_buffer *output, const ldns_key *k)
 {
@@ -1342,7 +1348,7 @@ ldns_hmac_key2buffer_str(ldns_buffer *output, const ldns_key *k)
 	ldns_buffer_printf(output, "\n");
 	return status;
 }
-
+#endif
 
 ldns_status
 ldns_key2buffer_str(ldns_buffer *output, const ldns_key *k)
@@ -1374,9 +1380,7 @@ ldns_key2buffer_str(ldns_buffer *output, const ldns_key *k)
 			case LDNS_SIGN_RSASHA1:
 			case LDNS_SIGN_RSASHA1_NSEC3:
 			case LDNS_SIGN_RSASHA256:
-			case LDNS_SIGN_RSASHA256_NSEC3:
 			case LDNS_SIGN_RSASHA512:
-			case LDNS_SIGN_RSASHA512_NSEC3:
 			case LDNS_SIGN_RSAMD5:
 				/* copied by looking at dnssec-keygen output */
 				/* header */
@@ -1405,27 +1409,20 @@ ldns_key2buffer_str(ldns_buffer *output, const ldns_key *k)
 								    "Algorithm: %u (RSASHA256)\n",
 								    LDNS_RSASHA256);
 					break;
-				case LDNS_SIGN_RSASHA256_NSEC3:
-					ldns_buffer_printf(output,
-								    "Algorithm: %u (RSASHA256_NSEC3)\n",
-								    LDNS_RSASHA256_NSEC3);
-					break;
 				case LDNS_SIGN_RSASHA512:
 					ldns_buffer_printf(output,
 								    "Algorithm: %u (RSASHA512)\n",
 								    LDNS_RSASHA512);
 					break;
-				case LDNS_SIGN_RSASHA512_NSEC3:
-					ldns_buffer_printf(output,
-								    "Algorithm: %u (RSASHA512_NSEC3)\n",
-								    LDNS_RSASHA512_NSEC3);
-					break;
 #endif
 				default:
 					fprintf(stderr, "Warning: unknown signature ");
 					fprintf(stderr,
 						   "algorithm type %u\n", 
 						   ldns_key_algorithm(k));
+					ldns_buffer_printf(output,
+								    "Algorithm: %u (Unknown)\n",
+								    ldns_key_algorithm(k));
 					break;
 				}
 				
@@ -1661,7 +1658,7 @@ ldns_key2buffer_str(ldns_buffer *output, const ldns_key *k)
 				break;
 			case LDNS_SIGN_HMACSHA256:
 		        ldns_buffer_printf(output, "Private-key-format: v1.2\n");
-		        ldns_buffer_printf(output, "Algorithm: 158 (HMAC_SHA256)\n");
+		        ldns_buffer_printf(output, "Algorithm: 159 (HMAC_SHA256)\n");
 				status = ldns_hmac_key2buffer_str(output, k);
 				break;
 		}
