@@ -52,12 +52,8 @@ ldns_rr_set_function(ldns_rr_type type, ldns_rr *rr, ldns_rdf *rdf, size_t pos)
                 return false;
         }
         pop = ldns_rr_set_rdf(rr, rdf, pos);
-        if (pop) {
- 		LDNS_FREE(pop);
-                return true;
-        } else {
-                return true;
-        }
+ 	ldns_rdf_deep_free(pop);
+        return true;
 }
 
 /* A/AAAA records */
@@ -295,9 +291,7 @@ ldns_rr_dnskey_key_size_raw(const unsigned char* keydata,
 	case LDNS_SIGN_RSASHA1_NSEC3:
 #ifdef USE_SHA2
 	case LDNS_SIGN_RSASHA256:
-	case LDNS_SIGN_RSASHA256_NSEC3:
 	case LDNS_SIGN_RSASHA512:
-	case LDNS_SIGN_RSASHA512_NSEC3:
 #endif
 		if (len > 0) {
 			if (keydata[0] == 0) {
